#!/usr/bin/env python
from powerline.lib.threaded import ThreadedSegment, with_docstring
import cjdnsadmin

def peers(pl):
    """Returns the current number of peers connected"""
    try:
        cjdns = cjdnsadmin.connectWithAdminInfo()
    except:
        return None
    peercount = 0
    localpeers = 0
    page = 0
    more = True
    while more:
        table = cjdns.InterfaceController_peerStats(page)
        more = "more" in table
        page += 1
        for peer in table['peers']:
            if peer['state'] != "UNRESPONSIVE":
                if "user" in peer:
                    if peer['user'] == "Local Peers":
                        localpeers += 1
                    else:
                        peercount += 1
                else:
                    peercount += 1
    out = [
            {
                "contents": str(peercount),
                "highlight_group": ["cjdns:peers"]
            }
        ]
    if localpeers > 0:
        out.append({
            "contents": str(localpeers),
            "highlight_group": ["cjdns:localpeers"]
            })
    return out

class NodesSegment(ThreadedSegment):
    interval = 5

    def update(self, oldnodes):
        try:
            cjdns = cjdnsadmin.connectWithAdminInfo()
        except:
            return None

        more = True
        page = 0
        routes = 0
        nodes = []
        while more:
            table = cjdns.NodeStore_dumpTable(page)
            more = "more" in table
            routes += len(table['routingTable'])
            for route in table['routingTable']:
                if not route['ip'] in nodes:
                    nodes.append(route['ip'])
            page += 1
        return nodes

    def render(self, nodes, **kwags):
        return [{
            "contents": str(len(nodes)),
            "highlight_group": ["cjdns:nodes"]
            }]
nodes = with_docstring(NodesSegment(),
        """Displays the number of nodes in the cjdns routing table""")

if __name__ == "__main__":
    print("Hint: If you see this message and you don't know why, try reading the included README.md file")
    print(peers(None))
    print(nodes(None))



#!/usr/bin/env python
from powerline.lib.threaded import ThreadedSegment, with_docstring
import requests

class BTCSegment(ThreadedSegment):
    interval = 300

    def set_state(self, exchange="bitstamp", currency="USD", **kwargs):
        self.exchange = exchange
        super(BTCSegment, self).set_state(**kwargs)

    def update(self, oldprice):
        price = None
        if self.exchange == "bitstamp":
            ticker = requests.get("https://www.bitstamp.net/api/ticker/").json()
            price = "$%s" % ticker['last']
        if price is not None:
            return price
        else:
            return "Unknown exchange \"%s\"" % exchange

    def render(self, price, **kwargs):
        return [{
            "contents": str(price),
            "hightlight_group": ["btc"]
            }]

btc = with_docstring(BTCSegment(),
"""Returns the current price of Bitcoin at the specified exchage
:param str exchange:
    The name of the exchange. Currently supported:

    * bitstamp (supported currencies: USD)

:param str currency:
    The currency being exchanged to. See above for available currencies for
    your exchange.
highlight groups: ``btc``
""")

class RedditMessageSegment(ThreadedSegment):
    interval = 60

    def set_state(self, username=None, password=None, **kwargs):
        self.headers = {"User-Agent": "Powerline Reddit Checker 0.1 (thefinn93@thefinn93.com)"}
        self.username = username
        self.password = password
        super(RedditMessageSegment, self).set_state(**kwargs)

    def dologin(self):
        login = requests.post("https://ssl.reddit.com/api/login", data = {
                "api_type": "json",
                "rem": "true",
                "user": self.username,
                "password": self.password}, headers = self.headers).json()
        errors = None
        cookies = None
        if "errors" in login:
            errors = login['errors']

        if "cookie" in login: # I'm hella ratelimited right now, can't test the actual value :(
            cookies = login['cookies']
        if "modhash" in login:
            self.headers['X-Modhash'] = login['modhash']

        return errors, cookies

    def update(self, oldstuff):
        errors = None
        olderrors = None
        cookies = None
        unread = None

        if oldstuff is None: # First run
            errors, cookies = self.dologin()
        else:
            olderrors, cookies, oldunread = oldstuff

            if cookies is None: # Not our first run, but no login cookies
                if olderrors[0][0] == "RATELIMIT": # we got ratelimited last time... try again
                    errors, cookies = self.dologin()
                elif olderrors[0][0] == "WRONG_PASSWORD": # Wrong password. Refuse to continue
                    return errors, cookies, None
        if cookies:
            unread = requests.get("https://ssl.reddit.com/message/unread.json",
                    params = {"mark": "false", "limit": 100},
                    cookies = cookies, headers = self.headers).json()

        return errors, cookies, unread

    def render(self, currentdata, **kwargs):
        errors, login, unread = currentdata
        ret = []
        if unread:
            if "data" in unread:
                if "children" in unread['data']:
                    if len(unread['data']['children']) > 0:
                        ret.append({
                            "contents": str(len(unread['data']['children'])),
                            "highlight_group": "reddit:messages"
                            })
                else:
                    raise Exception("UM WTF BRO", unread)
            else:
                raise Exception("UM WTF BRO", unread)

        if errors:
            ret.append({
                "contents": errors[0][1],
                "highlight_group": "reddit:messages"
                })
        return ret

redditmessages = with_docstring(RedditMessageSegment(),
        """Displays the number of unread reddit messages in your inbox.
:param str username
    Your reddit username

:param str password
    Your reddit password
""")



#!/usr/bin/env python
try:
    import apt
except ImportError:
    pass

from powerline.lib.threaded import ThreadedSegment, with_docstring


class AptUpgradesSegment(ThreadedSegment):
    interval = 300

    def update(self, oldthing):
        upgrades = 0
        for pkg in apt.Cache():
            if pkg.is_upgradable:
                upgrades += 1
        return upgrades
    def render(self, upgrades, **kwargs):
        if upgrades > 0:
            return [{
                "contents": "%i packages can be upgraded" % upgrades,
                "highlight_groups": ["packages:upgrades", "packages"]
                }]
        else:
            return None

AptUpgrades = with_docstring(AptUpgradesSegment(),
        """Displays the number of upgradable packages""")



from setuptools import setup, find_packages

setup(
    name='miniatureOctoBatman',
    packages=find_packages(),
    namespace_packages=['miniatureOctoBatman'],
    version='0.1.1',
    description="Finn's collection of powerline modules",
    author='Finn Herzfeld',
    author_email='finn@finn.io',
    url='https://github.com/thefinn93/miniature-octo-batman'
    )
